import os
import logging
from ..connector import Connector


class Client(Connector):
    """UNIX-socket client used to communicate with the daemon."""
    def __init__(self):
        Connector.__init__(self, 'unix', False)

    def connect(self, host):
        """
        Connects to the UNIX-socket if it exists.

        :return: if the connection to the socket was successfull
        :rType: bool
        """
        if os.path.exists(host):
            try:
                self.socket.connect(host)
                logging.debug('Successfully connected to UNIX socket at: %s' % host)
                return True
            except OSError as e:
                logging.error('Failed to connect to socket: %s' % e.strerror)
        else:
            logging.debug('Could not find socket: %s' % host)

        return False



import logging
from logging.handlers import RotatingFileHandler
from ..config import LOG_FILE_FORMAT, LOG_FILE_DATE_FORMAT, LOG_FILE_PATH, LOG_FILE_MAX_SIZE, LOG_FILE_COUNT


def create_rotating_log(name, log_level):
    """
    Creates a rotating log which will limit the size of the log file.

    :param name str: the name of the logger to add the rotating log handler to
    :param log_level int: the logging level to use, e.g logging.DEBUG
    """
    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    formatter = logging.Formatter(fmt=LOG_FILE_FORMAT, datefmt=LOG_FILE_DATE_FORMAT)
    handler = RotatingFileHandler(LOG_FILE_PATH, maxBytes=LOG_FILE_MAX_SIZE, backupCount=LOG_FILE_COUNT)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

def setup_logging(verbose, print_mode):
    """
    Creates a rotating log.

    :param verbose bool: if debug messages should be saved/printed
    :param print_mode bool: whether or not to print to the terminal
    """
    if verbose is True:
        if print_mode is True:
            logging.basicConfig(
                format=LOG_FILE_FORMAT,
                datefmt=LOG_FILE_DATE_FORMAT,
                level=logging.DEBUG
            )
        else:
            logging.basicConfig(
                format=LOG_FILE_FORMAT,
                datefmt=LOG_FILE_DATE_FORMAT,
                filename=LOG_FILE_PATH,
                level=logging.DEBUG,
                filemode='w' # since we are debugging we want to overwrite the log each time
            )
    else:
        create_rotating_log('', logging.ERROR) # use the root logger



import os
import sys
import atexit
import logging
import argparse
import subprocess

from .daemon import Daemon
from .utils.logger import setup_logging
from .config import DAEMON_VERSION, LOG_FILE_PATH

if sys.platform.startswith('win32'):
    from .channel.win.client import Client
else:
    from .channel.unix.client import Client

parser = argparse.ArgumentParser(description='Pywalfox - Native messaging host')
setup_group = parser.add_argument_group('install/uninstall')
start_group = parser.add_argument_group('start')
parser.add_argument('action',
        nargs='?',
        default=None,
        metavar='ACTION',
        help='available actions are install, start, update, log and uninstall')
parser.add_argument('-v', '--version',
        dest='version',
        action='store_true',
        help='displays the current version of the daemon')
start_group.add_argument('-p', '--print',
        dest='print_mode',
        action='store_true',
        help='writes debugging output from the native messaging host to stdout')
start_group.add_argument('--verbose',
        dest='verbose',
        action='store_true',
        help='runs the native messaging host in verbose mode')
setup_group.add_argument('-g', '--global',
        dest='global_install',
        action='store_true',
        help='installs/uninstalls the native host manifest globally')

def get_python_version():
    """Gets the current python version and checks if it is supported."""
    python_version = sys.version_info
    version_label = '%s.%s.%s' % (python_version[0], python_version[1], python_version[2])
    if python_version < (2, 7):
        logging.error('Python version %s is not supported' % version_label)
        sys.exit(1)
    else:
        logging.debug('Using python %s' % version_label)

    return python_version

def send_update_action():
    """Sends the update command to the socket server."""
    client = Client()

    for host in client.hosts:
        connected = client.connect(host)
        if connected is True:
            client.send_message('update')

def open_log_file():
    """Opens the daemon log file in an editor."""
    if os.path.isfile(LOG_FILE_PATH):
        if not sys.platform.startswith('win32'):
            editor = os.getenv('EDITOR', 'vi')
        else:
            editor = 'nano' # fallback

        subprocess.call([editor, LOG_FILE_PATH])
    else:
        print('No log file exists')

def print_version():
    """Prints the current version of the daemon."""
    print('v%s' % DAEMON_VERSION)

def run_daemon():
    """Starts the daemon."""
    daemon = Daemon(get_python_version().major)
    atexit.register(daemon.close)
    daemon.start()

def handle_args(args):
    """Handles CLI arguments."""
    if args.version:
        print_version()
        sys.exit(0)

    if args.action == 'update':
        send_update_action()
        sys.exit(0)

    if args.action == 'start':
        setup_logging(args.verbose, args.print_mode)
        run_daemon()

    if args.action == 'log':
        open_log_file()
        sys.exit(0)

    if args.action == 'install':
        from pywalfox.install import start_setup
        start_setup(args.global_install)
        sys.exit(0)

    if args.action == 'uninstall':
        from pywalfox.install import start_uninstall
        start_uninstall(args.global_install)
        sys.exit(0)

    # If no action was specified
    parser.print_help()

def main():
    """Application entry point."""
    args = parser.parse_args()
    handle_args(args)

if __name__ == '__main__':
    main()



import os
import socket
import logging
from ..config import UNIX_SOCKET_PATH, WIN_SOCKET_HOST, UNIX_SOCKET_PATH_ALT, WIN_SOCKET_HOST_ALT

class Connector:
    """
    Base class for the socket server and client.
    Depending on the current OS, a different socket type will be used,
    since UNIX-sockets are not properly supported on Windows.

    :param platform_id str: the current platform identifier, e.g. win32
    :param validate_host bool: check if the socket host is available before binding
    """
    def __init__(self, platform_id, validate_host=True):
        if platform_id == 'win32':
            if validate_host is True:
                self.host = self.get_win_socket_host()

            self.hosts = [WIN_SOCKET_HOST, WIN_SOCKET_HOST_ALT]
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            logging.debug('Setup socket server using AF_INET (win32)')
        else:
            if validate_host is True:
                self.host = self.get_unix_socket_path()

            self.hosts = [UNIX_SOCKET_PATH, UNIX_SOCKET_PATH_ALT]
            self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
            logging.debug('Setup socket server using AF_UNIX (linux/darwin)')

    def get_unix_socket_path(self):
        """
        Get an available path to bind the UNIX-socket to.

        :return: the path to be used when binding the UNIX-socket
        :rType: str
        """
        if os.path.exists(UNIX_SOCKET_PATH):
            logging.debug('Default UNIX-socket is already in use')
            return UNIX_SOCKET_PATH_ALT

        return UNIX_SOCKET_PATH

    def get_win_socket_host(self):
        """
        Get an available host and port to bind the UDP-socket to.

        :return: the host and port to be used when binding the UDP-socket
        :rType: (host, port)
        """
        is_valid = True
        test_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            test_socket.bind(WIN_SOCKET_HOST)
            test_socket.close()
        except OSError as e:
            is_valid = False
            if e.errno == 98: # errno 98 means that address is already bound
                logging.debug('Default UDP-socket host is already in use')
            else:
                logging.error('Failed to test UDP-socket host availability: %s' % str(e))

        if is_valid is True:
            return WIN_SOCKET_HOST

        return WIN_SOCKET_HOST_ALT

    def encode_message(self, message):
        """
        Encodes a message to be sent using the socket.

        :param message str: the message to encode into a bytestring
        :return: the encoded message
        :rType: bytestring
        """
        return message.encode('utf-8')

    def decode_message(self, raw):
        """
        Decodes a message received from the socket.

        :param raw bytestring: the raw message of bytes to decode
        :return: the decoded message
        :rType: str
        """
        return raw.decode('utf-8')

    def get_message(self):
        """
        Reads and decodes an incoming message.

        :return: the decoded data
        :rType: str
        """
        data = self.socket.recv(1024)
        if not data:
            logging.error('Failed to read data from socket')
            return

        return self.decode_message(data)

    def send_message(self, message):
        """
        Encodes and sends a message using the socket

        :param data str: the string to send
        """
        encoded_message = self.encode_message(message)
        self.socket.send(encoded_message)

    def close(self):
        """Closes the socket connection."""
        self.socket.close()



import os
import logging
from ..connector import Connector


class Server(Connector):
    """UNIX-socket server used to communicate with clients."""
    def __init__(self):
        Connector.__init__(self, 'unix')

    def delete_existing_socket(self):
        """Deletes the existing UNIX-socket if it exists."""
        if os.path.exists(self.host):
            os.remove(self.host)

    def start(self):
        """
        Binds the socket to file.

        :return: if the socket could be bound to the file
        :rType: bool
        """
        self.delete_existing_socket()
        try:
            self.socket.bind(self.host)
            logging.debug('Successfully bound socket to: %s' % self.host)
            return True
        except OSError as e:
            logging.error('Failed to create UNIX socket: %s' % e.strerror)

        return False

    def close(self):
        """Unbinds the socket and deletes the file."""
        self.socket.close()

        try:
            """
            UNIX-sockets can be overwritten by other processes even if another process
            is already using it. This may lead to the file not existing and will
            cause a crash if not handled properly.
            """
            os.remove(self.host)
            logging.debug('UNIX-socket deleted')
        except OSError as e:
            logging.debug('UNIX-socket has already been deleted, skipping')



import os
import sys
import shutil
import fileinput

from .config import APP_PATH, HOME_PATH, BIN_PATH_UNIX, BIN_PATH_WIN

if sys.platform.startswith('win32'):
    try:
        import _winreg as winreg
    except ImportError:
        import winreg

# We only need these variables when running the setup, so might as well define them here.
MANIFEST_SRC_PATH = os.path.join(APP_PATH, 'assets/manifest.json')
MANIFEST_TARGET_NAME = 'pywalfox.json'

WIN_REGISTRY_PATH = r'Software\Mozilla\NativeMessagingHosts\pywalfox'
MANIFEST_TARGET_PATH_WIN = os.path.join(HOME_PATH, '.pywalfox')

MANIFEST_TARGET_PATHS_LINUX = {
    'FIREFOX': os.path.join('/usr/lib/mozilla/native-messaging-hosts'),
    'FIREFOX_USER': os.path.join(HOME_PATH, '.mozilla/native-messaging-hosts'),
}

MANIFEST_TARGET_PATHS_DARWIN = {
    'FIREFOX': os.path.join('/Library/Application Support/Mozilla/NativeMessagingHosts'),
    'FIREFOX_USER': os.path.join(HOME_PATH, 'Library/Application Support/Mozilla/NativeMessagingHosts'),
}

def create_hosts_directory(hosts_path):
    """
    Creates the 'native-messaging-hosts' directory if it does not exist.

    :param hosts_path str: the path to create if it does not already exist
    """
    if not os.path.exists(hosts_path):
        os.makedirs(hosts_path)

def remove_existing_manifest(full_path, print_errors=True):
    """
    Removes an existing manifest.

    :param full_path str: the path to the manifest
    """
    try:
        if os.path.isfile(full_path):
            os.remove(full_path)
            print('Successfully removed manifest at: %s' % full_path)
        else:
            if print_errors is True:
                print('No manifest is installed at: %s' % full_path)
    except Exception as e:
        if print_errors is True:
            if e.errno == 13: # permission error
                print('Permission denied when trying to remove the manifest.')
                print('If you are trying to install it globally, rerun this script with admin privileges.')
                print('')
                print('If you installed Pywalfox for your user only, you must probably use something like this:')
                print('sudo python -m pywalfox uninstall')
            else:
                print('Could not remove existing manifest at: %s\n\t%s' % (full_path, str(e)))

            sys.exit(1)
        else:
            raise e

def normalize_path(target_path):
    """
    Replaces backslashes with forward slashes.

    :param target_path str: the path to normalize
    :return: the normalized path
    :rType: str
    """
    return (r'%s' % target_path).replace('\\', '/')

def get_full_manifest_path(target_path):
    return os.path.join(target_path, MANIFEST_TARGET_NAME)

def set_daemon_path(manifest_path, bin_path):
    """
    Replaces the '<path>' placeholder in the default manifest file with the path to the executable.

    :param manifest_path str: the path to the manifest
    :param bin_path str: the path to the daemon executable
    """
    normalized_path = normalize_path(bin_path)
    for line in fileinput.FileInput(manifest_path, inplace=1):
        line = line.replace("<path>", normalized_path)
        print(line.rstrip('\n'))

    print('Set daemon executable path to: %s' % normalized_path)

def copy_manifest(target_path, bin_path):
    """
    Copies the host manifest to the 'native-messaging-hosts' directory.

    :param target_path str: the path to the hosts directory
    :param bin_path str: the path to the daemon executable
    """
    full_path = get_full_manifest_path(target_path)

    try:
        create_hosts_directory(target_path)
        remove_existing_manifest(full_path, False)
        shutil.copyfile(MANIFEST_SRC_PATH, full_path)
        print('Copied manifest to: %s' % full_path)
    except Exception as e:
        if e.errno == 13: # permission error
            print('Permission denied when trying to install the manifest.')
            print('If you are trying to install it globally, rerun this script with admin privileges.')
            print('')
            print('If you installed Pywalfox for your user only, you must probably use something like this:')
            print('sudo python -m pywalfox install')
        else:
            print('Failed to install manifest: %s:\n%s' % (full_path, str(e)))

        sys.exit(1)

    set_daemon_path(full_path, bin_path)

def set_executable_permissions(bin_path):
    """
    Sets the execution permission on the daemon executable.
    https://stackoverflow.com/questions/12791997/how-do-you-do-a-simple-chmod-x-from-within-python

    :param bin_path str: the path to the daemon executable
    """
    try:
        mode = os.stat(bin_path).st_mode
        mode |= (mode & 0o444) >> 2    # copy R bits to X
        os.chmod(bin_path, mode)
        print('Set execute permissions on daemon executable')
    except Exception as e:
        print('Failed to set executable permissions on: %s\n\t%s' % (bin_path, str(e)))
        print('')
        print('Try setting the permissions manually using: chmod +x')
        sys.exit(1)

def get_target_path_key(global_install):
    """
    Gets the path key for the 'native-messaging-hosts' directory based on
    if the manifest should be installed locally or globally.

    :param global_install bool: if the manifest should be installed for all users
    :return: the key in MANIFEST_TARGET_PATHS_* corresponding to the manifest path
    :rType: str
    """
    if global_install is True:
        return 'FIREFOX'
    else:
        return 'FIREFOX_USER'

def setup_register(manifest_path_key):
    """
    Imports the winreg module and returns the hkey based on if the
    manifest should be installed for the current user only, or for all users.
    """
    hkey = winreg.HKEY_CURRENT_USER
    if manifest_path_key == 'FIREFOX':
        hkey = winreg.HKEY_LOCAL_MACHINE

    return hkey

def delete_registry_keys(manifest_path_key):
    """Tries to delete an existing registry key holding the path to the manifest."""
    hkey = setup_register(manifest_path_key)

    try:
        reg_key = winreg.OpenKey(hkey, WIN_REGISTRY_PATH, 0, winreg.KEY_SET_VALUE)
        print('Found existing registry key and opened with write permissions')
    except Exception:
        print('No existing registry key found')
        return

    try:
        winreg.DeleteValue(reg_key, '')
        winreg.DeleteKey(hkey, WIN_REGISTRY_PATH)
        print('Deleted registry key: %s' % WIN_REGISTRY_PATH)
    except Exception as e:
        print('Failed to remove existing registry key: %s' % str(e))
        return

def win_setup(manifest_path_key):
    """Windows installation."""
    hkey = setup_register(manifest_path_key)

    try:
        reg_key = winreg.OpenKey(hkey, WIN_REGISTRY_PATH, 0, winreg.KEY_SET_VALUE)
        print('Opened registry key with write permissions')
    except Exception:
        reg_key = winreg.CreateKey(hkey, WIN_REGISTRY_PATH)
        print('Created new registry key')

    try:
        normalized_target_path = normalize_path(os.path.join(MANIFEST_TARGET_PATH_WIN, MANIFEST_TARGET_NAME))
        winreg.SetValue(reg_key, '', winreg.REG_SZ, normalized_target_path)
        print('Set value of registry: %s to %s' % (WIN_REGISTRY_PATH, normalized_target_path))
    except Exception as e:
        print('Failed to set registry key: %s\n%s' % (reg_key, str(e)))
        sys.exit(1)

    copy_manifest(MANIFEST_TARGET_PATH_WIN, BIN_PATH_WIN)

def linux_setup(manifest_path_key):
    """Linux installation."""
    manifest_path = MANIFEST_TARGET_PATHS_LINUX[manifest_path_key]
    copy_manifest(manifest_path, BIN_PATH_UNIX)
    set_executable_permissions(BIN_PATH_UNIX)

def darwin_setup(manifest_path_key):
    """MacOS installation."""
    manifest_path = MANIFEST_TARGET_PATHS_DARWIN[manifest_path_key]
    copy_manifest(manifest_path, BIN_PATH_UNIX)
    set_executable_permissions(BIN_PATH_UNIX)

def validate_permissions(global_install):
    if os.geteuid() == 0 and global_install is False:
        print('You are running the script as root, but did not specify the --global option.')
        selection = input('Things may not work as expected, continue? (y/N): ')
        if selection.lower() != 'y' and selection.lower() != 'yes':
            sys.exit(1)

def start_setup(global_install):
    """
    Installs the native messaging host manifest.

    :param global_install bool: if the manifest should be installed for all users
    """
    validate_permissions(global_install)
    manifest_path_key = get_target_path_key(global_install)

    if sys.platform.startswith('win32'):
        win_setup(manifest_path_key)
    elif sys.platform.startswith('darwin'):
        darwin_setup(manifest_path_key)
    else:
        linux_setup(manifest_path_key)

def start_uninstall(global_install):
    """
    Tries to remove an existing manifest and delete registry keys (win32).

    :param global_install bool: if the manifest should be uninstalled for all users
    """
    validate_permissions(global_install)
    manifest_path_key = get_target_path_key(global_install)

    if sys.platform.startswith('win32'):
        manifest_path = MANIFEST_TARGET_PATH_WIN
        delete_registry_keys(manifest_path_key)
    elif sys.platform.startswith('darwin'):
        manifest_path = MANIFEST_TARGET_PATHS_DARWIN[manifest_path_key]
    else:
        manifest_path = MANIFEST_TARGET_PATHS_LINUX[manifest_path_key]

    full_manifest_path = get_full_manifest_path(manifest_path)
    remove_existing_manifest(full_manifest_path)



import os

DAEMON_VERSION = '2.7'

UNIX_SOCKET_PATH = '/tmp/pywalfox_socket'
UNIX_SOCKET_PATH_ALT = '/tmp/pywalfox_socket_alt'
WIN_SOCKET_HOST = ('127.0.0.1', 56744)
WIN_SOCKET_HOST_ALT = ('127.0.0.1', 56745)

SUPPORTED_BROWSERS = ['firefox']

HOME_PATH = os.path.expanduser('~')
XDG_CACHE_DIR = os.getenv('XDG_CACHE_HOME', os.path.join(HOME_PATH, '.cache'))
PYWAL_COLORS_PATH = os.path.join(XDG_CACHE_DIR, 'wal/colors.json')

APP_PATH = os.path.dirname(os.path.abspath(__file__))
CSS_PATH = os.path.join(APP_PATH, 'assets/css')
BIN_PATH_WIN = os.path.join(APP_PATH, 'bin/win.bat')
BIN_PATH_UNIX = os.path.join(APP_PATH, 'bin/main.sh')

FIREFOX_PROFILES_PATH_LINUX = os.path.join(HOME_PATH, '.mozilla/firefox')
FIREFOX_PROFILES_PATH_WIN = os.path.join(HOME_PATH, 'AppData/Roaming/Mozilla/Firefox')
FIREFOX_PROFILES_PATH_DARWIN = os.path.join(HOME_PATH, 'Library/Application Support/Firefox')

LOG_FILE_COUNT = 1
LOG_FILE_MAX_SIZE = 1000*200 # 0.2 mb
LOG_FILE_DATE_FORMAT = '%m-%d-%Y %I:%M:%S'
LOG_FILE_FORMAT = '[%(asctime)s] %(levelname)s:%(message)s'
LOG_FILE_PATH = os.path.join(XDG_CACHE_DIR, 'pywalfox.log')

ACTIONS = {
    'VERSION': 'debug:version',
    'OUTPUT': 'debug:output',
    'COLORS': 'action:colors',
    'INVALID_ACTION': 'action:invalid',
    'CSS_ENABLE': 'css:enable',
    'CSS_DISABLE': 'css:disable',
    'CSS_FONT_SIZE': 'css:font:size',
}



import sys
import json
import struct


class Messenger:
    """
    Handles the sending and receiving of messages to and from the extension using stdio.

    :param version str: the current major python version
    """
    def __init__(self, version):
        self.stdout, self.stdin = self.get_stdio_handle(version)

    def get_stdio_handle(self, python_version):
        """
        Gets the stdin and stdout handles depending on the current python version.
        Python 2.x uses 'sys.stdout.xxx', whereas python 3.x uses 'sys.stdout.buffer.xxx'.

        :return: (stdout handle, stdin handle) based on the current python version
        :rType: tuple
        """
        if python_version == 2:
            return (sys.stdout, sys.stdin)
        else:
            return (sys.stdout.buffer, sys.stdin.buffer)

    def decode_message(self, encoded_length):
        """
        Decodes a message received from stdin.

        :param encoded_length buffer: the buffer containing the data length
        :return: the decoded message
        :rType: object
        """
        data_length = struct.unpack('@I', encoded_length)[0]
        message = self.stdin.read(data_length).decode('utf-8')
        return json.loads(message)

    def encode_message(self, message):
        """
        Encodes a message to be sent to stdout.

        :param message object: the message to encode
        :return: (length of encoded message, encoded message)
        :rType: tuple
        """
        json_string = json.dumps(message).encode('utf-8')
        buffer_length = struct.pack('@I', len(json_string))
        return (buffer_length, json_string)

    def get_message(self):
        """
        Reads message from extension in stdin.

        :return: the decoded message
        :rType: str
        """
        encoded_length = self.stdin.read(4)
        if not encoded_length:
            sys.exit(0)

        return self.decode_message(encoded_length)

    def send_message(self, message_object):
        """
        Sends a message to stdout.

        :param message [Message|ErrorMessage]: the message to encode and send
        """
        length, encoded_message = self.encode_message(message_object.getMessage())
        self.stdout.write(length)
        self.stdout.write(encoded_message)
        self.stdout.flush()



import logging
import json
from json.decoder import JSONDecodeError
from .config import PYWAL_COLORS_PATH

def get_pywal_colors():
    """
    Fetches the Pywal colors from the cache file.

    :return: the colors generated by Pywal
    :rType: list
    """
    colors = []
    wallpaper = ''
    try:
        with open(PYWAL_COLORS_PATH, 'r') as f:
            pywal_data = json.load(f)
            if 'colors' in pywal_data:
                for key in pywal_data['colors']:
                    colors.append(pywal_data['colors'][key])
            else:
                error_message = '%s does not contain any color values' % PYWAL_COLORS_PATH
                logging.error(error_message)
                return (False, None, error_message)

            if 'wallpaper' in pywal_data:
                wallpaper = pywal_data['wallpaper']
            else:
                error_message = '%s does not contain a wallpaper path' % PYWAL_COLORS_PATH
                logging.error(error_message)
                return (False, None, error_message)

    except (IOError, JSONDecodeError):
        error_message = 'Could not read colors from: %s' % PYWAL_COLORS_PATH
        logging.error(error_message)
        return (False, None, error_message)

    if len(colors) < 16:
        error_message = '%s containing the generated Pywal colors is invalid' % PYWAL_COLORS_PATH
        logging.error(error_message)
        return (False, None, error_message)

    logging.debug('Successfully fetched Pywal colors')
    return (True, {'colors': colors, 'wallpaper': wallpaper}, None)



import logging


class Message:
    """
    Defines the structure of a generic message.

    :param action str: the action called by the extension
    :param data any: the additional data to send along with the message
    :param success bool: if the action was successfull
    :param error str: the error message if success is False
    """
    def __init__(self, action, data=None, success=True, message=None):
        self.action = action
        self.success = success
        self.data = data
        self.message = message

    def getMessage(self):
        """Creates the response message."""
        message = {
            'action': self.action,
            'success': self.success
        }

        if self.data is not None:
            message['data'] = self.data

        if self.message is not None:
            if self.success is True:
                message['message'] = self.message
            else:
                message['error'] = self.message

        logging.debug('Created message: %s' % message)
        return message
